package MT::Plugin::CustomPermalink;

use strict;

use MT;
use base qw(MT::Plugin);
use URI::Escape;

my $plugin = __PACKAGE__->new({
    id => 'custompermalink',
    name => 'Custom Permalink',
    author_name => 'Inspire-Design Co.ltd.',
    author_url => 'http://inspire-design.net/',
    description => '<__trans phrase="The permalink can be customized with regex-pattern.">',
    version => '0.3',
    settings => new MT::PluginSettings([
        [ 'searches', { Scope => 'blog' }],
        [ 'replaces', { Scope => 'blog' }],
        [ 'plugin_enable', { Scole => 'blog' }]
    ]),
    l10n_class => 'CustomPermalink::L10N',
    blog_config_template => 'blog_config.tmpl',
});
MT->add_plugin($plugin);

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        applications => {
            'cms' => {
                methods => {
                    custom_permalink_add_pair => '$CustomPermalink::CustomPermalink::CMS::add_pair',
                },
            },
        },
        tags => {
            function => {
                'CustomEntryPermalink' => '$CustomPermalink::CustomPermalink::_hdlr_custom_entry_permalink',
                'CustomPagePermalink' => '$CustomPermalink::CustomPermalink::_hdlr_custom_page_permalink',
                'CustomCategoryArchiveLink' => '$CustomPermalink::CustomPermalink::_hdlr_custom_category_archive',
                'CustomArchiveLink' => '$CustomPermalink::CustomPermalink::_hdlr_custom_archive_link',
            },
            modifier => {
                'custom_permalink' => '$CustomPermalink::CustomPermalink::_fltr_custom_permalink',
            }
        },
        callbacks => {
            'template_source.entry_table' => '$CustomPermalink::CustomPermalink::CMS::template_source_entry_table',
            'template_source.edit_entry' => '$CustomPermalink::CustomPermalink::CMS::template_source_edit_entry',
            'template_source.preview_strip' => '$CustomPermalink::CustomPermalink::CMS::template_source_preview_strip'
        },
    });
    my $core = MT->component('core');
    my $r = $core->registry('tags', 'function');
    $r->{EntryPermalink} = '$CustomPermalink::CustomPermalink::_hdlr_custom_entry_permalink';
    $r->{PagePermalink} = '$CustomPermalink::CustomPermalink::_hdlr_custom_page_permalink';
    $r->{CategoryArchiveLink} = '$CustomPermalink::CustomPermalink::_hdlr_custom_category_archive';
    $r->{ArchiveLink} = '$CustomPermalink::CustomPermalink::_hdlr_custom_archive_link';
}

sub debug {
    my $msg = shift;
    use Data::Dumper;
    open  FH, ">>debug.log";
    print FH Dumper($msg);
    close FH;
}

sub instance {
    return $plugin;
}

sub enabled {
    my $plugin = shift;
    my ($blog_id) = @_;
    my $enable = $plugin->get_config_value('plugin_enable', 'blog:' . $blog_id);

    return $enable ? 1 : 0;
}

sub replace {
    my $plugin = shift;
    my ($str) = @_;
    my $app = MT->instance;
    my $blog_id = $app->blog->id if $app->blog;

    return $plugin->translate("Invalid access.") if !$blog_id;

    my %param;
    $plugin->load_config(\%param, 'blog:' . $blog_id);

    for (my $i = 0; $i <= $#{$param{searches}}; $i++) {
        if (defined $param{replaces}[$i]) {
            eval($str =~ s/$param{searches}[$i]/$param{replaces}[$i]/);
        }
    }

    return $str;
}

sub save_config {
    my $plugin = shift;
    my ($args, $scope) = @_;
    my (@searches, @replaces);

    if ($scope =~ /blog:(\d+)/) {
        foreach my $key (%$args) {
            if ($key =~ /search_([0-9]+)/ && defined $args->{'replace_' . $1}) {
                $searches[$1] = uri_unescape($args->{'search_' . $1});
                $replaces[$1] = uri_unescape($args->{'replace_' . $1});
            }
        }

        $plugin->set_config_value('plugin_enable', $args->{'plugin_enable'}, $scope);
        $plugin->set_config_value('searches', \@searches, $scope);
        $plugin->set_config_value('replaces', \@replaces, $scope);
    }
}

sub load_config {
    my $plugin = shift;
    my ($args, $scope) = @_;

    $plugin->SUPER::load_config(@_);

    if ($scope =~ /blog:(\d+)/) {
        $args->{blog_id} = $1;
    }
}

1;